@@ -8,33 +8,12 @@ from json import JSONDecoder, JSONEncoder
 
 class MZNJSONEncoder(JSONEncoder):
     def default(self, o):
-        from .solver import Solver
-
-        if isinstance(o, set):
-            return {"set": [i for i in o]}
-        elif isinstance(o, range):
-            return {"set": [i for i in o]}
-        elif isinstance(o, Enum):
+        if isinstance(o, Enum):
             return {"e": o.name}
-        elif isinstance(o, Solver):
-            return {
-                "name": o.name,
-                "version": o.version,
-                "id": o.id,
-                "executable": o.executable,
-                "mznlib": o.mznlib,
-                "tags": o.tags,
-                "stdFlags": o.stdFlags,
-                "extraFlags": o.extraFlags,
-                "supportsMzn": o.supportsMzn,
-                "supportsFzn": o.supportsFzn,
-                "needsSolns2Out": o.needsSolns2Out,
-                "needsMznExecutable": o.needsMznExecutable,
-                "needsStdlibDir": o.needsStdlibDir,
-                "isGUIApplication": o.isGUIApplication,
-            }
-        else:
-            return super().default(o)
+        if isinstance(o, set) or isinstance(o, range):
+            return {"set": [{"e": i.name} if isinstance(i, Enum) else i for i in o]}
+
+        return super().default(o)
 
 
 class MZNJSONDecoder(JSONDecoder):
@@ -45,6 +24,16 @@ class MZNJSONDecoder(JSONDecoder):
             self.enum_map = enum_map
         JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)
 
+    def _lookup_enum(self, name: str):
+        if name in self.enum_map:
+            return self.enum_map[name]
+        else:
+            # TODO: mypy seems to believe name the elements should be literals,
+            # but I cannot find this anywhere in the documentation
+            anon_enum = Enum("AnonymousEnum", name)  # type: ignore
+            self.enum_map[name] = anon_enum(1)
+            return anon_enum(1)
+
     def object_hook(self, obj):
         if len(obj) == 1 and "set" in obj:
             if len(obj["set"]) == 1 and isinstance(obj["set"][0], list):
@@ -57,11 +46,11 @@ class MZNJSONDecoder(JSONDecoder):
                     assert len(item) == 2
                     li.extend([i for i in range(item[0], item[1] + 1)])
                 elif len(obj) == 1 and "e" in obj:
-                    li.append(self.enum_map.get(obj["e"], obj["e"]))
+                    li.append(self._lookup_enum(obj["e"]))
                 else:
                     li.append(item)
             return set(li)
         elif len(obj) == 1 and "e" in obj:
-            return self.enum_map.get(obj["e"], obj["e"])
+            return self._lookup_enum(obj["e"])
         else:
             return obj
